/* This program simulates the models for New Zealand GDP
    of the paper "Markov Switching Models for GDP Growth in a Small Open Economy:
    The New Zealand Experience", Buckle, Haugh, and Thompson
    Journal of Business cycle measurement and analysis                                  */

/* We simulate five models:
    1) AR(0) model
    2) AR(1) model
    3) Their estimates of simple MS model (Hamilton)
    4) Their model 2-2
    5) Their model 3-2
(see page 243 of the paper for parameter estimates                */

/* The simulation results are saved in "file1" which can be
    renamed at the bottom of the program                */
    
new;
library pgraph;
output file=temp.out reset;
declare u;
#include c:\mstest; 
rndseed 172618;

reps=63000;
toptrim=3000;

model1="AR(0)";
model2="AR(1)";
model3="MS(0)--Hamilton";
model4="MS(22)";
model5="MS(32)";


/* This may seem odd, but it's correct */
dataset="c:NZ_gdp.txt";
load u[26,4] = ^dataset;
u=vecr(u);
u=trimr(u,0,2);

t=rows(u);
m=sumc(u)/t;
v=sumc((u-m)^2)/(t-1); print "Mean and Variance of data"; print m~v;
print; print /flush "Press a key to continue";
wait;

/* This graph is currently turned off 
xy(seqa(1,1,rows(u)),u);
wait; */

/* changes=abs(u[2:rows(u)]-u[1:rows(u)-1]);
bigchanges=changes .>= sqrt(v);
"big changes" sumc(bigchanges .== 1);
"small changes" sumc(bigchanges .== 0);
highlevel=u[2:rows(u)] .>= m;
"high level" sumc(highlevel .== 1);
"small level" sumc(highlevel .== 0);
"big change, high level"  sumc((bigchanges .== 1) .* (highlevel .== 1)); 
"big change, low level"  sumc((bigchanges .== 1) .* (highlevel .== 0)); 
"small change, high level"  sumc((bigchanges .== 0) .* (highlevel .== 1)); 
"small change, low level"  sumc((bigchanges .== 0) .* (highlevel .== 0));  */

/* Parameters for linear, AR(0) model */
mu=.6;
sig=1.07;

/* Simulating and testing AR(0) model */
y=mu+(sig*rndn(reps,1));
y=trimr(y,toptrim,0);
dev=y-meanc(y);
dev1=lagn(dev,1);
dev=trimr(dev,1,0);
dev1=trimr(dev1,1,0);
print sumc(dev.*dev1)/rows(dev); 
ysavlat0=y;
mstest0(meanc(y),stdc(y)^2,u,dataset,model1);
mstest1(u,y,dataset,model1); 
mstest7(u,y,dataset,model1); 
print /flush "Strike a key to continue"; wait;

/* Parameters for linear, AR(1) model */
mu=.6;
sig=1.06;
rho=.14;

/* Simulating AR(1) model */
eps=sig*rndn(reps,1);
y0=0;
y=zeros(reps,1);
indx=1;
do while indx <= reps;
    y[indx]=((1-rho)*mu)+(y0*rho)+eps[indx];
    y0=y[indx];
    indx=indx+1;
    endo;
y=trimr(y,toptrim,0);
dev=y-meanc(y);
dev1=lagn(dev,1);
dev=trimr(dev,1,0);
dev1=trimr(dev1,1,0);
print sumc(dev.*dev1)/rows(dev); 
ysavlat1=y;
mstest0(meanc(y),stdc(y)^2,u,dataset,model2);
mstest1(u,y,dataset,model2); 
mstest7(u,y,dataset,model2); 
print /flush "Strike a key to continue"; wait;

/* Simple MS model (Hamilton) */
a0=0.07;
diff=1.01;
sigs=0.94;
p00=.82;
p11=.84;

/* Simulating the simple MS (Hamilton) model */
eps=sigs*rndn(reps,1);
strand=rndu(reps,1);
state=zeros(reps,1);
s0=1;                           /* start at state 1 (slightly more probable) */
indx=1;
do while indx <= reps;
    if s0==0;
        state[indx]= strand[indx] .> p00;
    else;
        state[indx]= strand[indx] .< p11;
    endif;
    s0=state[indx];
    indx=indx+1;
    endo;

z=a0+(diff*state)+eps;      /* z is the simulated data */
z=trimr(z,toptrim,0);

mstest0(meanc(z),stdc(z)^2,u,dataset,model3);
mstest1(u,z,dataset,model3);  
mstest7(u,z,dataset,model3); 
zsavlat2=z;
print /flush "Strike a key to continue"; wait;

/* 2-2 model  */
a0=0.44;
diff=0.65;
sig0=1.15;
sigdiff=-0.65;
p00=.97;
p11=.91;

/* Simulating the 2-2 model */
eps=rndn(reps,1);
strand=rndu(reps,1);
state=zeros(reps,1);
s0=0;                           /* start at state 0 (slightly more probable) */
indx=1;
do while indx <= reps;
    if s0==0;
        state[indx]= strand[indx] .> p00;
    else;
        state[indx]= strand[indx] .< p11;
    endif;
    s0=state[indx];
    indx=indx+1;
    endo;

z=a0+(diff*state)+(sig0+(sigdiff*state)).*eps;
z=trimr(z,toptrim,0);

mstest0(meanc(z),stdc(z)^2,u,dataset,model4);
mstest1(u,z,dataset,model4);  
mstest7(u,z,dataset,model4); 
zsavlat3=z;
print /flush "Strike a key to continue"; wait;

/* The 3-2 model  */
m1=1;
m2=0.23;
m3=1.81;
sig1=.42;
sig2=1.03;

p00=.94;
p11=.54;
q00=.91;
q11=.96;

/* Simulating the 3-2 model */

eps=rndn(reps,1);
tempdat=(m1+sig1*eps)~(m2+sig2*eps)~(m3+sig1*eps)~(m3+sig2*eps);
strand_p=rndu(reps,1);
strand_q=rndu(reps,1);
state_p=zeros(reps,1);
state_q=zeros(reps,1);

s0=0;                           /* start at state 0 (slightly more probable) */
indx=1;
do while indx <= reps;
    if s0==0;
        state_p[indx]= strand_p[indx] .> p00;
    else;
        state_p[indx]= strand_p[indx] .< p11;
    endif;
    s0=state_p[indx];
    indx=indx+1;
    endo;

s0=1;                           /* start at state 1 (slightly more probable) */
indx=1;
do while indx <= reps;
    if s0==0;
        state_q[indx]= strand_q[indx] .> q00;
    else;
        state_q[indx]= strand_q[indx] .< q11;
    endif;
    s0=state_q[indx];
    indx=indx+1;
    endo;

state=((state_q .== 0).*(state_p .== 0))+(2*(state_q .== 1).*(state_p .== 0))+
        (3*(state_q .== 0).*(state_p .== 1))+(4*(state_q .== 1).*(state_p .== 1));

print "Here are the proportion of time spent in the various states";
print "state1" sumc(state .== 1)/rows(state);
print "state2" sumc(state .== 2)/rows(state);
print "state3" sumc(state .== 3)/rows(state);
print "state4" sumc(state .== 4)/rows(state);
wait;

z=tempdat[1,state[1]];
indx=2;
do while indx <= reps;
    z=z|tempdat[indx,state[indx]];
    indx=indx+1;
    endo;

z=trimr(z,toptrim,0);

mstest0(meanc(z),stdc(z)^2,u,dataset,model5);
mstest1(u,z,dataset,model5);  
mstest7(u,z,dataset,model5); 
zsavlat4=z;
print /flush "Strike a key to continue"; wait;


file1="c:\simdata0";
let vnames=ar0sim ar1sim ms0sim ms22sim ms32sim;
create f1=^file1 with ^vnames,0,8;
writer(f1,ysavlat0~ysavlat1~zsavlat2~zsavlat3~zsavlat4);  

end;

